﻿var timespans = { oneSecond: 1000, oneMinute: 60 * 1000, oneHour: 60 * 60 * 1000, oneDay: 24 * 60 * 60 * 1000, oneWeek: 7 * 24 * 60 * 60 * 1000, oneMonth: 31 * 24 * 60 * 60 * 1000 };
var fetchIntervalDaily = 2 * timespans.oneHour;
var fetchIntervalIntraday = 1 * timespans.oneHour;
var fetchIntervalSingleDay = 1 * timespans.oneMinute;
var fetchInterval = 200;
var fetchTimer = 0;
var currencyFormatter = new Intl.NumberFormat('en-GB', { style: 'currency', currency: 'GBP' });
var instrumentSearchResults = [];

var Instruments = {
  Data: [],
  indexOfSymbol: function (Symbol) { return this.Data.map(function (item) { return item.Symbol; }).indexOf(Symbol); },
  GetSymbol: function (Symbol) { return this.Data[this.indexOfSymbol(Symbol)]; },
  GetHolding: function (holdingID) {
    for (let x = 0; x < this.Data.length; x++) {
      let i = this.Data[x];
      for (let n = 0; n < i.Holdings.length; n++) {
        let h = i.Holdings[n];
        if (h.HoldingID == holdingID) {
          return { instrument: i, holding: h};
        }
      }
    }
  },
  GetCurrentHoldings: function (Instrument) {
    let result = [];
    for (let x = 0; x < Instrument.Holdings.length; x++) {
      let h = Instrument.Holdings[x];
      if (h.SoldDate == null) {
        result.push(h);
      }
    }
    return result;
  },
  GetLastFetchedDaily: function () {
    if (!this.Data[0].lastFetchedDaily) { this.Data[0].lastFetchedDaily = this.Data[0].MaxDailyDate };
    let lastFetchedDate = this.Data[0].lastFetchedDaily;
    let lastFetchedIndex = 0;
    for (let i = 1; i < this.Data.length; i++) {
      if (!this.Data[i].lastFetchedDaily) { this.Data[i].lastFetchedDaily = this.Data[i].MaxDailyDate };
      if (lastFetchedDate > this.Data[i].lastFetchedDaily) {
        lastFetchedDate > this.Data[i].lastFetchedDaily;
        lastFetchedIndex = i;
      }
    }
    return this.Data[lastFetchedIndex];
  },
  GetLastFetchedIntraday: function () {
    if (!this.Data[0].lastFetchedIntraday) { this.Data[0].lastFetchedIntraday = this.Data[0].MaxIntradayDate };
    let lastFetchedDate = this.Data[0].lastFetchedIntraday;
    let lastFetchedIndex = 0;
    for (let i = 1; i < this.Data.length; i++) {
      if (!this.Data[i].lastFetchedIntraday) { this.Data[i].lastFetchedIntraday = this.Data[i].MaxIntradayDate };
      if (lastFetchedDate > this.Data[i].lastFetchedIntraday) {
        lastFetchedDate > this.Data[i].lastFetchedIntraday;
        lastFetchedIndex = i;
      }
    }
    return this.Data[lastFetchedIndex];
  },
  GetLastFetchedSingleDay: function () {
    if (!this.Data[0].lastFetchedSingleDay) { this.Data[0].lastFetchedSingleDay = 0 };
    let lastFetchedDate = this.Data[0].lastFetchedSingleDay;
    let lastFetchedIndex = 0;
    for (let i = 1; i < this.Data.length; i++) {
      if (!this.Data[i].lastFetchedSingleDay) { this.Data[i].lastFetchedSingleDay = 0 };
      if (lastFetchedDate > this.Data[i].lastFetchedSingleDay) {
        lastFetchedDate > this.Data[i].lastFetchedSingleDay;
        lastFetchedIndex = i;
      }
    }
    return this.Data[lastFetchedIndex];
  },
  GetCurrentPrice: function (Instrument) {
    if (Instrument.CurrentPrice) {
      return Instrument.CurrentPrice;
    } else {
      for (let i = Instrument.IntradayData.length - 1; i >= 0; i--) {
        if (Instrument.IntradayData[i].close) {
          return Instrument.IntradayData[i].close;
        }
      }
      for (let i = Instrument.DailyData.length - 1; i >= 0; i--) {
        if (Instrument.DailyData[i].close) {
          return Instrument.DailyData[i].close;
        }
      }
    }
  },
  GetExchangeRate: function (fromCurrency, toCurrency) {
    if (fromCurrency == 'GBp' && toCurrency == 'GBP') {
      return 100;
    } else {
      let curr = this.GetSymbol(toCurrency + fromCurrency + '=X');
      if (curr) {
        //return curr.CurrentPrice;
        return this.GetCurrentPrice(curr);
      } else {
        curr = this.GetSymbol(toCurrency + fromCurrency + '=X');
        if (curr) {
          if (curr.CurrentPrice) {
            //return 1/curr.CurrentPrice;
            return 1/this.GetCurrentPrice(curr);
          }
        }
      }
    }
  },
  MarketIsOpen: function (instrument) {
    let currentDT = new Date().getTime();
    let startOfDay = new Date().setHours(0, 0, 0);
    if (instrument.SingleDayStartDate > (startOfDay + timespans.oneDay)) {
      currentDT += timespans.oneDay;
    }
    /*
    if (instrument.Symbol == 'QAN.AX') {
      console.info('Qantas');
      console.info('  Current local time: ' + new Date().getTime().toString() + new Date().yyyymmddhhmmss());
      console.info('  SingleDayStartDate: ' + new Date(instrument.SingleDayStartDate).getTime().toString() + new Date(instrument.SingleDayStartDate).yyyymmddhhmmss());
      console.info('  SingleDayEndDate: ' + new Date(instrument.SingleDayEndDate).getTime().toString() + new Date(instrument.SingleDayEndDate).yyyymmddhhmmss());
    }
    */
    if (currentDT < instrument.SingleDayStartDate) {
      return 0; //Not open yet
    } else {
      if (currentDT > instrument.SingleDayEndDate) {
        return 2; //Market closed (end of day)
      } else {
        return 1; //Market is currently open
      }
    }
  }
}

Date.prototype.yyyymmddhhmmss = function () {
  var yyyy = this.getFullYear().toString();
  var mm = (this.getMonth() + 1).toString(); // getMonth() is zero-based         
  var dd = this.getDate().toString();
  var hh = this.getHours().toString();
  var nn = this.getMinutes().toString();
  var ss = this.getSeconds().toString();
  return yyyy + '-' + (mm[1] ? mm : "0" + mm[0]) + '-' + (dd[1] ? dd : "0" + dd[0]) + " " + (hh[1] ? hh : "0" + hh[0]) + ":" + (nn[1] ? nn : "0" + nn[0]) + ":" + (ss[1] ? ss : "0" + ss[0]);
};
Date.prototype.yyyymmdd = function () {
  var yyyy = this.getFullYear().toString();
  var mm = (this.getMonth() + 1).toString(); // getMonth() is zero-based         
  var dd = this.getDate().toString();
  return yyyy + '-' + (mm[1] ? mm : "0" + mm[0]) + '-' + (dd[1] ? dd : "0" + dd[0]);
};
Date.prototype.hhmm = function () {
  var hh = this.getHours().toString();
  var nn = this.getMinutes().toString();
  return (hh[1] ? hh : "0" + hh[0]) + ":" + (nn[1] ? nn : "0" + nn[0]);
};

function logInfo(msg) {
  console.info(msg);
  $("#logDiv").css("font-color", "white").text(msg);
}
function logWarning(msg) {
  console.warn(msg);
  $("#logDiv").css("font-color", "orange").text(msg);
}
function logError(msg) {
  console.error(msg);
  $("#logDiv").css("font-color", "red").text(msg);
}
function formatAmount(amount, currency, decimals) {
  let isNegative = (amount < 0);
  let s = Math.abs(amount).toFixed(decimals);
  let pos = s.indexOf('.');
  let wholeNumber = (pos >= 0) ? s.substring(0, pos) : s;
  let decimal = (pos >= 0) ? s.substring(pos) : '';
  s = '';
  while (wholeNumber.length > 0) {
    //if (s.length > 0 && wholeNumber!='-') s = ',' + s; //Don't add a comma if the only thing left to add to the string is the minus sign
    if (s.length > 0) s = ',' + s;
    if (wholeNumber.length >= 3) {
      s = wholeNumber.substring(wholeNumber.length - 3) + s;
      wholeNumber = wholeNumber.substring(0, wholeNumber.length - 3);
    } else {
      s = wholeNumber + s;
      wholeNumber = '';
    }
  }
  s = s + decimal;
  let sign = isNegative ? '-' : '';
  switch (currency) {
    case 'GBP':
      s = sign + '£' + s;
      break;
    case 'GBp':
      s = sign + s + 'p';
      break;
    case 'USD':
      s = sign + 'U$' + s;
      break;
    case 'AUD':
      s = sign + 'A$' + s;
      break;
    case 'EUR':
      s = sign + '€' + s;
      break;
    default:
      s = sign + currency + s;
  }
  return s;
}
function getProfitOrLoss(BasePrice, CurrentPrice, ShowPercent) {
  let priceMovement = (CurrentPrice - BasePrice).toFixed(2);
  let percentMovement = (((CurrentPrice / BasePrice) - 1) * 100).toFixed(1) + '%';
  if (percentMovement[0] == '-') percentMovement = percentMovement.substring(1);
  return '<span class="' + (CurrentPrice < BasePrice ? 'loss' : 'profit') + '">' + (CurrentPrice < BasePrice ? '' : '+') + priceMovement + (ShowPercent ? ' (' + percentMovement + ')' : '') + '</span>';
}

function getInstruments() {
  if (fetchTimer != 0) { clearTimeout(fetchTimer) };
  //console.info('getInstruments()');
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/GetInstruments",
    dataType: "json",
    success: function (response) {
      //console.info('getInstruments Success');
      //Instruments.Data = response;
      Instruments.Data = response.Instruments;
      Instruments.Accounts = response.Accounts;
      for (let i = 0; i < Instruments.Data.length; i++) {
        Instruments.Data[i].Holdings = [];
        Instruments.Data[i].HoldingsHistory = [];
      }
      let symbol = '';
      let instrument = {};
      for (let i = 0; i < response.Holdings.length; i++) {
        if (symbol != response.Holdings[i].Symbol) {
          symbol = response.Holdings[i].Symbol;
          instrument = Instruments.GetSymbol(response.Holdings[i].Symbol);
        }
        instrument.Holdings.push(response.Holdings[i]);
      }
      for (let i = 0; i < response.HoldingsHistory.length; i++) {
        let h = response.HoldingsHistory[i];
        if (symbol != h.Symbol) {
          symbol = h.Symbol;
          instrument = Instruments.GetSymbol(h.Symbol);
        }
        instrument.HoldingsHistory.push(h);
      }
      createSharesTable();
    },
    failure: function (response) { console.error("GetInstruments error: " + response.d) }
  });
};

function createSharesTable() {
  //console.info('createSharesTable()');
  var tbl = '<table id="tblInstruments"><tr><th>Instrument</th>' +
                                           '<th>YTD</th>' +
                                           '<th>1 Month<button onclick="setMid3Month">3 Month</button>' +
                                                       '<button onclick="setMid2Month">2 Month</button>' +
                                                       '<button onclick="setMid1Month">1 Month</button></th>' +
                                           '<th>1 Week</th>' +
                                           '<th>1 Day</th></tr>';
  for (var rowNo = 0; rowNo < Instruments.Data.length; rowNo++) {
    let i = Instruments.Data[rowNo];
    if (i.DisplayOrder) {
      //let cls = (rowNo % 2) ? 'altShareRow' : 'shareRow';
      let cls = (rowNo % 2) ? 'shareRow altShareRow' : 'shareRow';
      let tr = '<tr class="' + cls + '" id="shareRow' + i.DisplayOrder.toString() + '">' + createSharesTableRow(i) + '</tr>';
      tbl += tr;
    }
  }
  tbl += '</table>';
  $('#chartDiv').html(tbl);
  //$('#chartDiv span.addHolding').unbind().bind('click', function () { showModalDialog_AddHolding(this) });
  bindShareRowEvents();
  getAllLocalDailyData();
};
function createSharesTableRow(instrument) {
  function getHoldingsTable(Instrument) {
    if (Instrument.Holdings.length > 0) {
      let showPreviousHoldings = $('#showPreviousHoldings').prop('checked');
      let totalUnits = 0;
      let totalPrice = 0;
      let noRows = 0;
      let t = '<table class="miniHoldings"><tr><th>Account</th><th>Buy Date</th><th>Price</th><th>Units</th><th>Total</th></tr>';
      for (let i = 0; i < Instrument.Holdings.length; i++) {
        let h = Instrument.Holdings[i];
        if (h.SoldDate == null) {
          noRows++;
          totalUnits += h.NoUnits;
          totalPrice += (h.NoUnits * h.PurchasePricePerUnit);
          t += '<tr><td><span class="deletebutton" onclick="showModalDialog_DeleteHolding(' + h.HoldingID + ')">&times;</span>' + h.AccountName + '</td>' +
                   '<td>' + new Date(h.PurchaseDate).yyyymmdd() + '</td>' +
                   '<td class="num">' + formatAmount(h.PurchasePricePerUnit, '', 2) + '</td>' +
                   '<td class="num">' + formatAmount(h.NoUnits, '', 0) + '</td>' +
                   '<td class="num">' + formatAmount(h.NoUnits * h.PurchasePricePerUnit, '', 0) + '</td></tr>';
        } else {
          if (showPreviousHoldings) {
            noRows++;
            t += '<tr class="soldHolding"><td>' + h.AccountName + '</td>' +
                     '<td>' + new Date(h.PurchaseDate).yyyymmdd() + '</td>' +
                     '<td class="num">' + formatAmount(h.PurchasePricePerUnit, '', 2) + '</td>' +
                     '<td class="num">' + formatAmount(h.NoUnits, '', 0) + '</td>' +
                     '<td class="num">' + formatAmount(h.NoUnits * h.PurchasePricePerUnit, '', 0) + '</td></tr>';
          }
        }
      }
      //if (Instrument.Holdings.length > 1) {
      if (totalUnits > 0) {
        Instrument.TotalUnitsHeld = totalUnits;
        Instrument.TotalHoldingsCost = totalPrice;
        Instrument.BreakevenPrice = totalPrice / totalUnits;
        t += '<tr><td class="no-border"></td>' +
                 '<td class="no-border">Total</td>' +
                 '<td class="num">' + formatAmount(Instrument.BreakevenPrice, '', 2) + '</td>' +
                 '<td class="num">' + formatAmount(Instrument.TotalUnitsHeld, '', 0) + '</td>' +
                 '<td class="num">' + formatAmount(Instrument.TotalHoldingsCost, '', 0) + '</td>' +
             '</tr>';
      }
      t += '</table>';
      return (noRows>0) ? t : '';
    } else {
      delete Instrument.TotalUnitsHeld;
      delete Instrument.TotalHoldingsCost;
      delete Instrument.BreakevenPrice;
      return '';
    }
  }
  
  let dispOrder = instrument.DisplayOrder.toString();
  return '<td>' + instrument.InstrumentName + '<br>' +
           '<a target="_blank" href="https://uk.finance.yahoo.com/quote/' + instrument.Symbol + '">' + instrument.Symbol + '</a>&nbsp;<span class="addHolding" data-symbol="' + instrument.Symbol + '">+</span><div class="spacer"></div>' +
           getHoldingsTable(instrument) +
           '<div class="price-summary" id="divCurrentValue' + dispOrder + '" />' +
         '</td>' +
         '<td><div class="LongChart" id="divLongChart' + dispOrder + '"></div><div class="LongSummary" id="divLongSummary' + dispOrder + '" /></td>' +
         '<td><div class="MidChart" id="divMidChart' + dispOrder + '"></div><div class="MidSummary" id="divMidSummary' + dispOrder + '" /></td>' +
         '<td><div class="ShortChart" id="divShortChart' + dispOrder + '"></div><div class="ShortSummary" id="divShortSummary' + dispOrder + '" /></td>' +
         '<td><div class="DayChart" id="divDayChart' + dispOrder + '"></div><div class="DaySummary" id="divDaySummary' + dispOrder + '" /></td>'
};
function bindShareRowEvents() {
  $('#chartDiv span.addHolding').unbind().bind('click', function () { showModalDialog_AddHolding(this) });
  /*
  $('#tblInstruments .shareRow').unbind().hover(function () {
    showShareRowPriceSummaries(this);
  }, function () {
    hideShareRowPriceSummaries(this);
  });
  */
}
function showShareRowPriceSummaries(row) {
  $(row).find('price-summary, .LongSummary, .MidSummary, .ShortSummary, .DaySummary').css('display', 'inherit');
}
function hideShareRowPriceSummaries(row) {
  $(row).find('price-summary, .LongSummary, .MidSummary, .ShortSummary, .DaySummary').css('display', 'none');
}

function redrawAllSharesCharts() {
  for (n = 0; n < Instruments.Data.length; n++) {
    let i = Instruments.Data[n];
    createLongChart(i);
    createMidChart(i);
    createShortChart(i);
    createSingleDayChart(i);
  }
}

function getNewRemoteData() {
  clearTimeout(fetchTimer);
  let lastFetchedDailyInstrument = Instruments.GetLastFetchedDaily();
  if (lastFetchedDailyInstrument.lastFetchedDaily < (new Date().getTime() - fetchIntervalDaily)) {
    //console.info(new Date().yyyymmddhhmmss() + ": Last fetched daily: " + lastFetchedDailyInstrument.Symbol + ", Last fetched date: " + new Date(lastFetchedDailyInstrument.lastFetchedDaily).yyyymmddhhmmss());
    //logInfo(new Date().yyyymmddhhmmss() + ": Last fetched daily: " + lastFetchedDailyInstrument.Symbol + ", Last fetched date: " + new Date(lastFetchedDailyInstrument.lastFetchedDaily).yyyymmddhhmmss());
    getYahooDailyData(lastFetchedDailyInstrument);
  } else {
    let lastFetchedIntradayInstrument = Instruments.GetLastFetchedIntraday();
    if (lastFetchedIntradayInstrument.lastFetchedIntraday < (new Date().getTime() - fetchIntervalIntraday)) {
      //console.info(new Date().yyyymmddhhmmss() + ": Last fetched intraday: " + lastFetchedIntradayInstrument.Symbol + ", Last fetched date: " + new Date(lastFetchedIntradayInstrument.lastFetchedIntraday).yyyymmddhhmmss());
      //logInfo(new Date().yyyymmddhhmmss() + ": Last fetched intraday: " + lastFetchedIntradayInstrument.Symbol + ", Last fetched date: " + new Date(lastFetchedIntradayInstrument.lastFetchedIntraday).yyyymmddhhmmss());
      getYahooIntradayData(lastFetchedIntradayInstrument);
    } else {
      let lastFetchedSingleDayInstrument = Instruments.GetLastFetchedSingleDay();
      if (lastFetchedSingleDayInstrument.lastFetchedSingleDay < (new Date().getTime() - fetchIntervalSingleDay)) {
        //console.info(new Date().yyyymmddhhmmss() + ": Last fetched single day: " + lastFetchedSingleDayInstrument.Symbol + ", Last fetched date: " + new Date(lastFetchedSingleDayInstrument.lastFetchedSingleDay).yyyymmddhhmmss());
        //logInfo(new Date().yyyymmddhhmmss() + ": Last fetched single day: " + lastFetchedSingleDayInstrument.Symbol + ", Last fetched date: " + new Date(lastFetchedSingleDayInstrument.lastFetchedSingleDay).yyyymmddhhmmss());
        getYahooSingleDayData(lastFetchedSingleDayInstrument);
      } else {
        fetchInterval = 500;
      }
    }
  }

  fetchTimer = setTimeout(getNewRemoteData, fetchInterval);
}
function getAllLocalDailyData() {
  let startDate = new Date(new Date().getFullYear(), 0, 1);
  //let endDate = new Date(new Date().setHours(0, 0, 0, 0));

  // Set start date to 2020-01-01 to include pre-pandemic prices
  startDate = new Date("2020-01-01 00:00:00");

  let endDate = new Date(new Date());
  let params = { startDate: startDate.getTime(), endDate: endDate.getTime() };
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/GetAllDailyData",
    dataType: "json",
    data: JSON.stringify(params),
    success: function (response) {
      for (let x = 0; x < response.length; x++) {
        let i = Instruments.GetSymbol(response[x].Symbol);
        i.DailyData = response[x].DailyData;
      }
      getAllLocalIntradayData();
    },
    failure: function (response) { console.error("getYTDData error:"); console.info(response); }
  });
}
function getAllLocalIntradayData() {
  let startDate = new Date(new Date().getFullYear(), 0, 1);
  //let endDate = new Date(new Date().setHours(0, 0, 0, 0));
  let endDate = new Date(new Date());

  // Set start date to 2020-01-01 to include pre-pandemic prices
  startDate = new Date("2020-01-01 00:00:00");

  let params = { startDate: startDate.getTime(), endDate: endDate.getTime() };
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/GetAllIntradayData",
    dataType: "json",
    data: JSON.stringify(params),
    success: function (response) {
      for (let x = 0; x < response.length; x++) {
        let i = Instruments.GetSymbol(response[x].Symbol);
        i.IntradayData = response[x].IntradayData;
        if (i.DailyData.length > 0) {
          createMidChart(i);
          createLongChart(i);
        }
        if (i.IntradayData.length > 0) {
          createShortChart(i);
        }
      }
      fetchTimer = setTimeout(getNewRemoteData, 500);
    },
    failure: function (response) { console.error("getYTDData error:"); console.info(response); }
  });
}

function createLongChart(Instrument) {
  let rowNo = Instrument.DisplayOrder;
  let ytdStartDate = new Date(new Date().getFullYear(), 0, 1).getTime();

  // Set start date to 2020-01-01 to include pre-pandemic prices
  ytdStartDate = new Date("2020-01-01 00:00:00");

  let ytdEndDate = new Date().getTime();
  createChart(Instrument, ytdStartDate, ytdEndDate, 'divLongChart' + rowNo.toString(), 'divLongSummary' + rowNo.toString());
}
function createMidChart(Instrument) {
  let rowNo = Instrument.DisplayOrder;
  let monthStartDate = new Date().setHours(0, 0, 0) - timespans.oneMonth;
  let monthEndDate = new Date().getTime();
  //createChart(Instrument, monthStartDate, monthEndDate, 'divMidChart' + rowNo.toString(), 'tdMidSummary' + rowNo.toString());
  createChart(Instrument, monthStartDate, monthEndDate, 'divMidChart' + rowNo.toString(), 'divMidSummary' + rowNo.toString());
}
function createShortChart(Instrument) {
  let rowNo = Instrument.DisplayOrder;
  let weekStartDate = new Date().setHours(0, 0, 0) - timespans.oneWeek;
  let weekEndDate = new Date().getTime();
  createChart(Instrument, weekStartDate, weekEndDate, 'divShortChart' + rowNo.toString(), 'divShortSummary' + rowNo.toString());
}
function createSingleDayChart(Instrument) {
  let rowNo = Instrument.DisplayOrder;
  let dayStartDate = 0;
  let dayEndDate = 0;
  let dt = new Date().getTime();
  //if (dt >= Instrument.SingleDayStartDate && dt <= Instrument.SingleDayEndDate) { //Market is currently open
    dayStartDate = Instrument.SingleDayStartDate;
    dayEndDate = Instrument.SingleDayEndDate;
  //} else { //Market is closed
    //dayStartDate = Instrument.SingleDayData[0].DT;
    //dayEndDate = Instrument.SingleDayData[Instrument.SingleDayData.length - 1].DT;
  //}
  let previousClose = Instrument.SingleDayPreviousClose;
  createChart(Instrument, dayStartDate, dayEndDate, 'divDayChart' + rowNo.toString(), 'divDaySummary' + rowNo.toString(), previousClose);
}
function createChart(Instrument, startDate, endDate, chartContainerID, summaryContainerID, previousClose) {
  function getChartDataSubset(Instrument, dateFrom, dateTo) {
    let result = [];
    //Single day data can be passed direct
    if (dateTo - dateFrom <= timespans.oneDay) {
      result = Instrument.SingleDayData;
      //return Instrument.SingleDayData;
    } else {
      //Anything over one month should only use daily data
      if (dateTo - dateFrom > (timespans.oneMonth + (timespans.oneDay*4))) {
        for (let i = 0; i < Instrument.DailyData.length; i++) {
          let quote = Instrument.DailyData[i];
          if (quote.DT >= dateFrom && quote.DT <= dateTo) {
            result.push(quote);
          }
        }
      } else {
        //Get all in-date-range intraday data
        let foundDates = { min: new Date().getTime, max: 0 }
        for (let i = 0; i < Instrument.IntradayData.length; i++) {
          let quote = Instrument.IntradayData[i];
          if (quote.DT >= dateFrom && quote.DT <= dateTo) {
            if (quote.DT > foundDates.max) foundDates.max = quote.DT;
            if (quote.DT < foundDates.min) foundDates.min = quote.DT;
            result.push(quote);
          }
        }
        //Back-fill with daily data if the intraday data doesn't cover the whole date range
        let minFoundDate = new Date(foundDates.min).setHours(0, 0, 0);
        if (minFoundDate > dateFrom) {
          for (let i = 0; i < Instrument.DailyData.length; i++) {
            let quote = Instrument.DailyData[i];
            if (quote.DT >= dateFrom && quote.DT < minFoundDate) {
              result.push(quote);
            } else {
              break;
            }
          }
        }
      }
    }
    return result;
  }
  function getBreakevenLine(Instrument, dateFrom, dateTo) {
    let result = [];
    let currentPrice = 0;
    for (let i = 0; i < Instrument.HoldingsHistory.length; i++) {
      let h = Instrument.HoldingsHistory[i];
      if (currentPrice > 0) { result.push([h.DT - 1, currentPrice]) };
      if (h.TotalUnits > 0) {
        currentPrice = h.TotalCost / h.TotalUnits;
        result.push([h.DT, (h.TotalCost / h.TotalUnits)]);
      } else {
        currentPrice = 0;
        result.push([h.DT, null]);
      }
    }
    if (currentPrice > 0) {
      result.push([dateTo, currentPrice]);
    }

    /*
    if (Instrument.Symbol == 'FLT.AX') {
      console.info("Before removing:");
      for (let i = 0; i < result.length; i++) {
        console.info(new Date(result[i][0]).yyyymmddhhmmss() + ' = ' + String(result[i][1]));
      }
    }
    */
    //Remove all points before the start of the date range
    let i = result.length - 1;
    while (i >= 0 && result[i][0] >= startDate) i--;
    while (i > 0) { result.shift(); i--; }
    /*
    if (Instrument.Symbol == 'FLT.AX') {
      console.info("After removing:");
      for (let i = 0; i < result.length; i++) {
        console.info(new Date(result[i][0]).yyyymmddhhmmss() + ' = ' + String(result[i][1]));
      }
    }
    */

    return result;
  }
  function transformAndSummariseChartData(cd) {
    let seriesData = [];
    //Find the first entry within the date range
    let x = 0;
    let summaryData = {
      StartDT: cd[x].DT,
      EndDT: cd[cd.length - 1].DT,
      Open: cd[x].open,
      High: cd[x].high,
      HighDT: cd[x].DT,
      Low: cd[x].low,
      LowDT: cd[x].DT,
      Close: cd[cd.length - 1].close
    };
    while (x < cd.length && cd[x].DT < startDate) {
      summaryData.StartDT = cd[x].DT;
      summaryData.EndDT = cd[cd.length - 1].DT;
      summaryData.Open = cd[x].open;
      summaryData.High = cd[x].high;
      summaryData.HighDT = cd[x].DT;
      summaryData.Low = cd[x].low;
      summaryData.LowDT = cd[x].DT;
      summaryData.Close = cd[cd.length - 1].close;
      x++;
    };
    let basePrice = (!previousClose) ? summaryData.Open : previousClose;
    for (let i = 0; i < cd.length; i++) {
      let entry = cd[i];
      if (summaryData.High < entry.high) {
        summaryData.High = entry.high;
        summaryData.HighDT = cd[x].DT;
      }
      if (summaryData.Low > entry.low) {
        summaryData.Low = entry.low;
        summaryData.LowDT = cd[x].DT;
      }
      if (i >= x) {
        if (priceType == 'percent') {
          seriesData.push([entry.DT, ((entry.close / basePrice) - 1) * 100]);
        } else {
          seriesData.push([entry.DT, entry.close]);
        }
      }
    }
    return { summaryData: summaryData, seriesData: seriesData, basePrice: basePrice };
  }
  function zzz_transformAndSummariseChartData(cd) {
      let seriesData = [];
      //Find the first entry within the date range
      let x = 0;
      let summaryData = {
        StartDT: cd[x].DT,
        EndDT: cd[cd.length - 1].DT,
        Open: cd[x].open,
        High: cd[x].high,
        Low: cd[x].low,
        Close: cd[cd.length - 1].close
      };
      while (x < cd.length && cd[x].DT < startDate) {
        summaryData.StartDT = cd[x].DT;
        summaryData.EndDT = cd[cd.length - 1].DT;
        summaryData.Open = cd[x].open;
        summaryData.High = cd[x].high;
        summaryData.Low = cd[x].low;
        summaryData.Close = cd[cd.length - 1].close;
        x++;
      };
      let basePrice = (!previousClose) ? summaryData.Open : previousClose;
      for (let i = 0; i < cd.length; i++) {
        let entry = cd[i];
        if (summaryData.High < entry.high) summaryData.High = entry.high;
        if (summaryData.Low > entry.low) summaryData.Low = entry.low;
        if (i >= x) {
          if (priceType == 'percent') {
            seriesData.push([entry.DT, ((entry.close / basePrice) - 1) * 100]);
          } else {
            seriesData.push([entry.DT, entry.close]);
          }
        }
      }
      return { summaryData: summaryData, seriesData: seriesData, basePrice: basePrice };
    }
  /*function getWeekendMarkings(startDate, endDate) {
    var markings = [],
    d = new Date(startDate);

    //Get the first Saturday
    d.setUTCDate(d.getUTCDate() - ((d.getUTCDay() + 1) % 7))
    d.setUTCSeconds(0);
    d.setUTCMinutes(0);
    d.setUTCHours(0);
    var i = d.getTime();

    do {
      markings.push({ xaxis: { from: i, to: i + 2 * timespans.oneDay }, color: '#e8e8e8' });
      i += 7 * timespans.oneDay;
    } while (i < endDate);

    return markings;
  }*/

  let priceType = $('#displayAsPercent').prop('checked') ? 'percent' : '';
  //let chartData = getChartDataSubset(Instrument, startDate, endDate);
  let chartData = [];
  if (endDate - endDate <= timespans.oneDay) {
    chartData = getChartDataSubset(Instrument, startDate, endDate);
  } else {
    chartData = getChartDataSubset(Instrument, startDate - (timespans.oneDay * 3), endDate);
  }
  
  //console.info(new Date().yyyymmddhhmmss() + ' - createChart ' + Instrument.Symbol + ' ' + chartContainerID + ' ' + new Date(startDate).yyyymmddhhmmss() + ' -> ' + new Date(endDate).yyyymmddhhmmss());
  let seriesData = [];
  if (chartData.length > 1) {
    /*let transformedAndSummarisedData = transformAndSummariseChartData(chartData);
    let summaryData = transformedAndSummarisedData.summaryData;
    let basePrice = transformedAndSummarisedData.basePrice;
    seriesData = transformedAndSummarisedData.seriesData;*/
    
    //Summarise the data and transform values to percentage if required
    let summaryData = {
      StartDT: chartData[0].DT,
      EndDT: chartData[chartData.length - 1].DT,
      Open: chartData[0].open,
      High: chartData[0].high,
      HighDT: chartData[0].DT,
      Low: chartData[0].low,
      LowDT: chartData[0].DT,
      Close: chartData[chartData.length - 1].close
    };
    let basePrice = (!previousClose) ? summaryData.Open : previousClose;
    for (let i = 0; i < chartData.length; i++) {
      let entry = chartData[i];
      if (summaryData.High < entry.high) {
        summaryData.High = entry.high;
        summaryData.HighDT = entry.DT;
      }
      if (summaryData.Low > entry.low) {
        summaryData.Low = entry.low;
        summaryData.LowDT = entry.DT;

      }
      if (priceType == 'percent') {
        seriesData.push([entry.DT, ((entry.close / basePrice) - 1) * 100]);
      } else {
        seriesData.push([entry.DT, entry.close]);
      }
    }
        
    //let weekendMarkings = getWeekendMarkings(startDate, endDate);
    let color = (summaryData.Close > basePrice) ? "green" : (summaryData.Close < basePrice) ? "red" : "blue";
    series = [{ data: seriesData, color: color, shadowSize: 0, lines: { lineWidth: 1 } }];

    //Add the breakeven price line
    if ($('#showBreakevenLine').prop('checked')) {
      if (priceType == 'percent') {
        series.unshift({ data: [[startDate, ((Instrument.BreakevenPrice / basePrice) - 1) * 100], [endDate, ((Instrument.BreakevenPrice / basePrice) - 1) * 100]], color: '#24BCC7', shadowSize: 0, lines: { lineWidth: 1 } });
      } else {
        //series.unshift({ data: [[startDate, Instrument.BreakevenPrice], [endDate, Instrument.BreakevenPrice]], color: '#24BCC7', shadowSize: 0, lines: { lineWidth: 1 } });
        let b = getBreakevenLine(Instrument, startDate, endDate);
        if (b.length > 0 && b[0][0]<endDate && b[b.length-1][0]>startDate) { //Only add the series if it falls within the chart date range
          //series.unshift({ data: b, color: '#00cc00', shadowSize: 0, lines: { lineWidth: 1.5 } });
          series.push({ data: b, color: '#00cc00', shadowSize: 0, lines: { lineWidth: 1.5 } });
        }
      }
    }

    //Add the previousClose line to the series
    if (previousClose) {
      if (priceType == 'percent') {
        series.unshift({ data: [[startDate, ((previousClose / basePrice) - 1) * 100], [endDate, ((previousClose / basePrice) - 1) * 100]], color: 'blue', shadowSize: 0, lines: { lineWidth: 1 } });
      } else {
        series.unshift({ data: [[startDate, previousClose], [endDate, previousClose]], color: 'blue', shadowSize: 0, lines: { lineWidth: 1 } });
      }
    }
    
    let chartTimespan = endDate - startDate;
    let gapMap = [];
    if (chartTimespan > timespans.oneDay && chartTimespan < timespans.oneMonth * 3) {
      gapMap = generateChartGapMap(seriesData, timespans.oneHour);
    }
    $.plot($('#' + chartContainerID), series, {
      grid: { show: true, hoverable: true /*, markings: weekendMarkings*/ },
      /*crosshair: { mode: 'x', snapX: true, showXValue: false },*/
      xaxis: {
        mode: "time",
        timezone: 'browser',
        min: chartTimespan > timespans.oneDay ? summaryData.StartDT : startDate,
        /*min: startDate, */
        max: chartTimespan > timespans.oneDay ? summaryData.EndDT : endDate,
        /*max: endDate,*/
        transform: function (v) { return mapChartXValue(v, gapMap); },
        inverseTransform: function (v) { return unmapChartXValue(v, gapMap); }
      }
    });
    
    //Bind the chart tooltip
    $("#" + chartContainerID).bind("plothover", function (event, pos, item) {
      if (!pos.x || !pos.y) { return; }
      if (item) {
        let x = new Date(item.datapoint[0]).yyyymmddhhmmss();
        let y = item.datapoint[1].toFixed(2);
        let tt = $("#chartTooltip");
        tt.html(x + " = " + y).css({ top: item.pageY + 5, left: item.pageX + 5 }).fadeIn(200);
      } else {
        $("#chartTooltip").hide();
      }
    });
    $("#" + chartContainerID).bind("mouseout", function (event, pos, item) {
      $("#chartTooltip").hide();
    });

    //Create the summary table
    let summaryTable = '<table class="summary">';
    if ((endDate - startDate) <= timespans.oneDay) {
      let dt = new Date().getTime();
      let marketHours = (new Date(Instrument.SingleDayStartDate)).hhmm() + ' - ' + (new Date(Instrument.SingleDayEndDate)).hhmm();
      summaryTable += '<tr><td>' + ((dt >= Instrument.SingleDayStartDate && dt <= Instrument.SingleDayEndDate) ? '<span style="color:limegreen">Market Open' : '<span style="color:firebrick">Market Closed') + '</span> (' + marketHours + ')</td>';
      summaryTable += (!previousClose) ? '' : ('<td colspan="2">Prev close: ' + previousClose.toString() + '</td>');
      summaryTable += '</tr>';
    }
    summaryTable += '<tr><td>O: ' + summaryData.Open.toFixed(2) + ' ' + ((previousClose) ? getProfitOrLoss(previousClose, summaryData.Open, 1) : '') + '</td>';
    summaryTable += '<td title="' + new Date(summaryData.HighDT).yyyymmddhhmmss() + '">H: ' + summaryData.High.toFixed(2) + ' ' + getProfitOrLoss(basePrice, summaryData.High, 1) + '</td><tr>';
    summaryTable += '<tr><td title="' + new Date(summaryData.LowDT).yyyymmddhhmmss() + '">L: ' + summaryData.Low.toFixed(2) + ' ' + getProfitOrLoss(basePrice, summaryData.Low, 1) + '</td>';
    summaryTable += '<td>C: ' + summaryData.Close.toFixed(2) + ' ' + getProfitOrLoss(basePrice, summaryData.Close, 1) + '</tr></td>';
    summaryTable += '</table>';
    $('#' + summaryContainerID).html(summaryTable);
  } else {
    //console.warn('Not enough data to plot ' + Instrument.Symbol + ' ' + new Date(startDate).yyyymmddhhmmss() + ' -> ' + new Date(endDate).yyyymmddhhmmss() + ' to ' + chartContainerID);
  }
}

function getYahooDailyData(Instrument) {
  //console.info('getYahooDailyData: ' + Instrument.Symbol);
  Instrument.lastFetchedDaily = new Date().getTime();
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/FetchYahooFinanceDaily",
    dataType: "json",
    data: JSON.stringify({ Symbol: Instrument.Symbol }),
    success: function (response) {
      let newData = [];
      if (response.chart.result) {
        let dataSeries = response.chart.result[0];
        let nonSubmittedEntries = 0;
        for (let i = 0; i < dataSeries.timestamp.length; i++) {
          let quote = dataSeries.indicators.quote[0];
          if (quote.open[i] == null || quote.high[i] == null || quote.low[i] == null || quote.close[i] == null || quote.volume[i] == null) {
            //console.info("Not submitting entry with null values: " + JSON.stringify({ DT: dataSeries.timestamp[i] * 1000, open: quote.open[i], high: quote.high[i], low: quote.low[i], close: quote.close[i], volume: quote.volume[i] }));
            nonSubmittedEntries += 1;
          } else {
            newData.push({ DT: dataSeries.timestamp[i] * 1000, open: quote.open[i], high: quote.high[i], low: quote.low[i], close: quote.close[i], volume: quote.volume[i] });
          }
        }
        if (nonSubmittedEntries > 0) {
          //console.warn("Didn't submit " + nonSubmittedEntries.toString() + " entries (" + newData.length.toString() + " OK) with null values for " + Instrument.Symbol);
        }

        //let d = new Date(newData[newData.length - 1].DT).yyyymmddhhmmss();
        //console.info("getYahooDailyData: " + Instrument.Symbol + ', Last data point: ' + d);

        submitNewDailyData(Instrument, newData);
        createMidChart(Instrument);
        createLongChart(Instrument);
      } else {
        console.info("No data received for symbol: " + Instrument.Symbol);
      }
    },
    failure: function (response) { console.error("getYahooDailyData error:"); console.info(response); }
  });
}
function getYahooIntradayData(Instrument) {
  //console.info('getYahooIntradayData: ' + Instrument.Symbol);
  Instrument.lastFetchedIntraday = new Date().getTime();
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/FetchYahooFinanceIntraday",
    dataType: "json",
    data: JSON.stringify({ Symbol: Instrument.Symbol }),
    success: function (response) {
      let newData = [];
      if (response.chart.result) {
        let dataSeries = response.chart.result[0];
        let nonSubmittedEntries = 0;
        for (let i=0; i<dataSeries.timestamp.length; i++) {
          let quote = dataSeries.indicators.quote[0];
          if (quote.open[i] == null || quote.high[i] == null || quote.low[i] == null || quote.close[i] == null || quote.volume[i] == null) {
            nonSubmittedEntries += 1;
            //console.info("Not submitting entry with null values: " + JSON.stringify({ DT: dataSeries.timestamp[i] * 1000, open: quote.open[i], high: quote.high[i], low: quote.low[i], close: quote.close[i], volume: quote.volume[i] }));
          } else {
            newData.push({DT: dataSeries.timestamp[i]*1000, open: quote.open[i], high: quote.high[i], low: quote.low[i], close: quote.close[i], volume: quote.volume[i]});
          }
        }
        if (nonSubmittedEntries > 0) {
          //console.warn("Didn't submit " + nonSubmittedEntries.toString() + " entries (" + newData.length.toString() + " OK) with null values for " + Instrument.Symbol);
        }
        submitNewIntradayData(Instrument, newData);
        createMidChart(Instrument);
        createShortChart(Instrument);
      } else {
        console.info("No data received for symbol: " + Instrument.Symbol);
      }
    },
    failure: function (response) { console.error("getYahooIntradayData error:"); console.info(response); }
  });  
}
function getYahooSingleDayData(Instrument) {
  //console.info('getYahooSingleDayData: ' + Instrument.Symbol);
  Instrument.lastFetchedSingleDay = new Date().getTime();
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/FetchYahooFinanceSingleDay",
    dataType: "json",
    data: JSON.stringify({ Symbol: Instrument.Symbol }),
    success: function (response) {
      let newData = [];
      if (response.chart.result) {
        if (response.chart.result[0].timestamp) {
          let dataSeries = response.chart.result[0];
          let ignoredEntries = 0;
          for (let i = 0; i < dataSeries.timestamp.length; i++) {
            let quote = dataSeries.indicators.quote[0];
            if (quote.open[i] == null || quote.high[i] == null || quote.low[i] == null || quote.close[i] == null || quote.volume[i] == null) {
              ignoredEntries++;
            } else {
              newData.push({ DT: dataSeries.timestamp[i] * 1000, open: quote.open[i], high: quote.high[i], low: quote.low[i], close: quote.close[i], volume: quote.volume[i] });
            }
          }
          if (ignoredEntries > 0) {
            //console.warn("Ignored " + ignoredEntries.toString() + " entries (" + newData.length.toString() + " OK) with null values for " + Instrument.Symbol);
          }

          Instrument.SingleDayData = newData;
          Instrument.SingleDayPreviousClose = dataSeries.meta.previousClose;
          Instrument.SingleDayOpenPrice = dataSeries.indicators.quote[0].open[0];

          Instrument.SingleDayStartDate = dataSeries.meta.currentTradingPeriod.regular.start * 1000;
          Instrument.SingleDayEndDate = dataSeries.meta.currentTradingPeriod.regular.end * 1000;

          Instrument.InstrumentType = dataSeries.meta.instrumentType;
          Instrument.Currency = dataSeries.meta.currency;

          Instrument.CurrentPrice = dataSeries.meta.regularMarketPrice;
          let currentValue = '';
          if (Instrument.CurrentPrice) {
            if (Instrument.TotalUnitsHeld > 0) {
              currentValue = '<div class="spacer"></div>';
              currentValue += 'Price (' + Instrument.Currency + '): ' + Instrument.BreakevenPrice.toFixed(2) + ' -&gt; ' + Instrument.CurrentPrice.toFixed(2) + ': ' + getProfitOrLoss(Instrument.BreakevenPrice, Instrument.CurrentPrice) + '<br>';
              currentValue += 'Value: ' + (Instrument.TotalHoldingsCost).toFixed(2) + ' -&gt; ' + (Instrument.TotalUnitsHeld * Instrument.CurrentPrice).toFixed(2) + ': ' + getProfitOrLoss(Instrument.TotalHoldingsCost, Instrument.TotalUnitsHeld * Instrument.CurrentPrice);
              let exRate = Instruments.GetExchangeRate(Instrument.Currency, 'GBP');
              if (exRate) {
                if (Instrument.Currency != 'GBp') {
                  currentValue += '<br>GBP/' + Instrument.Currency + ' = ' + exRate.toFixed(4);
                }
                currentValue += '<div class="current-value spacer">';
                //currentValue += 'Current value:<br>' + '£' + (Instrument.TotalUnitsHeld * Instrument.CurrentPrice / exRate).toFixed(2) + ': £' + getProfitOrLoss(Instrument.TotalHoldingsCost / exRate, Instrument.TotalUnitsHeld * Instrument.CurrentPrice / exRate);
                currentValue += 'Current value:<br>' + currencyFormatter.format(Instrument.TotalUnitsHeld * Instrument.CurrentPrice / exRate) + ': ' + getProfitOrLoss(Instrument.TotalHoldingsCost / exRate, Instrument.TotalUnitsHeld * Instrument.CurrentPrice / exRate, 1);
                currentValue += '</div>';
              } else {
                currentValue += '<br>' + Instrument.Currency;
              }
            } else {
              //INDEX, CURRENCY, EQUITY
              if (Instrument.InstrumentType == 'CURRENCY') {
                currentValue = 'Exchange Rate: ' + Instrument.CurrentPrice.toFixed(4);
              } else {
                currentValue = 'Price (' + Instrument.Currency + '): ' + Instrument.CurrentPrice.toFixed(2);
              }
            }
          }
          $('#divCurrentValue' + Instrument.DisplayOrder).html(currentValue);

          createSingleDayChart(Instrument);
          createHoldingsTable();
        } else {
          //console.warn("No timestamp series received for symbol: " + Instrument.Symbol);
          logWarning("No timestamp series received for symbol: " + Instrument.Symbol);
        }
      } else {
        //console.info("No data received for symbol: " + Instrument.Symbol);
        logInfo("No data received for symbol: " + Instrument.Symbol);
      }
    },
    failure: function (response) { console.error("getYahooSingleDayData error:"); console.info(response); }
  });
}

function submitNewDailyData(Instrument, NewData) {
  //console.info("submitNewDailyData");
  let currentMaxDT = Instrument.DailyData.length ? Instrument.DailyData[Instrument.DailyData.length - 1].DT : 0;
  for (let i = 0; i < NewData.length; i++) {
    if (NewData[i].DT > currentMaxDT) {
      Instrument.DailyData.push(NewData[i]);
    }
  }
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/SubmitDailyData",
    dataType: "json",
    data: JSON.stringify({Symbol: Instrument.Symbol, DailyPrices: NewData}),
    success: function (response) {
      //console.info('SubmitDailyData success: ' + Instrument.Symbol + ' - ' + JSON.stringify(response));
    },
    failure: function (response) { console.error("SubmitDailyData error: " + JSON.stringify(response)); }
  });
}
function submitNewIntradayData(Instrument, NewData) {
  //console.info("submitNewIntradayData");
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/SubmitIntradayData",
    dataType: "json",
    data: JSON.stringify({ Symbol: Instrument.Symbol, DailyPrices: NewData }),
    success: function (response) {
      //console.info('SubmitIntradayData success: ' + Instrument.Symbol + ' - ' + JSON.stringify(response));
    },
    failure: function (response) { console.error("SubmitIntradayData error: " + JSON.stringify(response)); }
  });

  //Add new data to Instrument
  let currentSummary = {};
  if (!Instrument.IntradayData || Instrument.IntradayData.length<1) {
    Instrument.IntradayData = [];
    currentSummary = { minDate: new Date().getTime(), maxDate: 0 };
  } else {
    currentSummary = { minDate: Instrument.IntradayData[0].DT, maxDate: Instrument.IntradayData[Instrument.IntradayData.length-1].DT };
  }

  let dataToAddToFront = [];
  for (let i = 0; i < NewData.length; i++) {
    if (NewData[i].DT < currentSummary.minDate) {
      dataToAddToFront.push(NewData[i]);
    } else {
      if (NewData[i].DT > currentSummary.maxDate) Instrument.IntradayData.push(NewData[i]);
    }
  }
  for (let i = dataToAddToFront.length - 1; i >= 0; i--) {
    Instrument.IntradayData.unshift(dataToAddToFront[i]);
  }
}

function createHoldingsTable() {
  function calcTotalHoldingsValue() {
    let totalValue = 0;
    for (let n = 0; n < Instruments.Data.length; n++) {
      let i = Instruments.Data[n];
      try {
        let c = Instruments.GetCurrentHoldings(i);
        if (i.CurrentPrice) {
          let exchangeRate = Instruments.GetExchangeRate(i.Currency, 'GBP');
          for (let x = 0; x < c.length; x++) {
            let h = c[x];
            let currentValue = h.NoUnits * (i.Currency == 'GBp' ? i.CurrentPrice / 100 : i.CurrentPrice);
            totalValue += currentValue;
          }
        }
      }
      catch (err) {
        console.info("Error adding " + i.Symbol + " to main holdings table: " + err.message);
      }
    }
    return totalValue;
  }

  let holdingCurrencies = {
    data: [],
    indexOfHoldingCurrency: function (symbol) { return this.data.map(function (item) { return item.symbol; }).indexOf(symbol); },
    getHoldingCurrency: function (symbol) { return this.data[this.indexOfHoldingCurrency(symbol)]; },
    addHoldingCurrency: function (symbol, amount) {
      let c = this.getHoldingCurrency(symbol);
      if (c) {
        c.total += amount;
      } else {
        this.data.push({ symbol: symbol, total: amount });
      }
    },
    getChartData: function () {
      let d = []
      for (i = 0; i < this.data.length; i++) {
        d.push({ label: this.data[i].symbol, data: this.data[i].total / Instruments.GetExchangeRate(this.data[i].symbol, 'GBP') });
      }
      return d;
    }
  };
  let totalHoldingsValue = calcTotalHoldingsValue();

  let tbl = '<table id="tblMainHoldings" class="mainHoldings"><thead>' +
                  '<tr><th>Name</th>' +
                  '<th>Symbol</th>' +
                  '<th>Account</th>' +
                  '<th>Buy Date</th>' +
                  '<th>No Units</th>' +
                  '<th>Buy<br>Price</th>' +
                  '<th>Current<br>Price</th>' +
                  '<th>Book<br>Cost</th>' +
                  '<th>Current<br>Value</th>' +
                  '<th>Gain</th>' +
                  '<th>Gain%</th>' +
                  '<th>Current<br>Value £</th>' +
                  '<th>Allocation</th>' +
                  '<th>Gain £</th>' +
                  '<th>Today&apos;s<br>Gain £</th>' +
                  '<th>Today&apos;s<br>Gain %</th>' +
              '</tr></thead><tbody>';
  let altRow = 1;
  let totalBookCostGBP = 0;
  let totalValueGBP = 0;
  let totalGainGBP = 0;
  let todaysGainGBP = 0;
  for (let n = 0; n < Instruments.Data.length; n++) {
    let i = Instruments.Data[n];
    try {
      let c = Instruments.GetCurrentHoldings(i);
      if (c.length > 0) { altRow = !altRow };
      if (i.CurrentPrice) {
        for (let x = 0; x < c.length; x++) {
          let h = c[x];
          let profitOrLoss = i.CurrentPrice < h.PurchasePricePerUnit ? 'loss' : 'profit';
          let marketIsOpen = Instruments.MarketIsOpen(i);
          let exchangeRate = Instruments.GetExchangeRate(i.Currency, 'GBP');
          let bookCost = h.NoUnits * (i.Currency == 'GBp' ? h.PurchasePricePerUnit / 100 : h.PurchasePricePerUnit);
          let currentValue = h.NoUnits * (i.Currency == 'GBp' ? i.CurrentPrice / 100 : i.CurrentPrice);
          let gain = (i.Currency == 'GBp' ? (i.CurrentPrice - h.PurchasePricePerUnit) / 100 : (i.CurrentPrice - h.PurchasePricePerUnit)) * h.NoUnits;
          let gainPercent = (i.CurrentPrice - h.PurchasePricePerUnit) / h.PurchasePricePerUnit * 100;
          let currentValueGBP = (h.NoUnits * i.CurrentPrice) / exchangeRate;
          let gainGBP = ((i.CurrentPrice - h.PurchasePricePerUnit) * h.NoUnits) / exchangeRate;
          let singleDayGainGBP = ((i.CurrentPrice - i.SingleDayPreviousClose) * h.NoUnits) / exchangeRate;
          let singleDayGainPercent = ((i.CurrentPrice / i.SingleDayPreviousClose) - 1) * 100;
          let singleDayProfitOrLoss = i.CurrentPrice < i.SingleDayPreviousClose ? 'loss' : 'profit';

          holdingCurrencies.addHoldingCurrency(i.Currency, h.NoUnits * h.PurchasePricePerUnit);

          totalBookCostGBP += h.NoUnits * h.PurchasePricePerUnit / exchangeRate;
          totalValueGBP += currentValueGBP;
          totalGainGBP += gainGBP;
          todaysGainGBP += (marketIsOpen == 0 ? 0 : singleDayGainGBP);

          tbl += '<tr' + (altRow ? ' class="altShareRow"' : '') + '>';
          //Group holdings of same shares
          /*if (c.length > 1) {
            if (x == 0) {
              tbl += '<td rowspan="' + c.length.toString() + '">' + i.InstrumentName + '</td>';
              tbl += '<td rowspan="' + c.length.toString() + '"><a target="_blank" href="https://uk.finance.yahoo.com/quote/' + i.Symbol + '">' + i.Symbol + '</a>' + '</td>';
            }
          } else { */
            tbl += '<td>' + i.InstrumentName + '</td>';
            tbl += '<td><a target="_blank" href="https://uk.finance.yahoo.com/quote/' + i.Symbol + '">' + i.Symbol + '</a>' + '</td>';
          //}
          tbl += '<td>' + h.AccountName + '</td>' +
                  '<td>' + new Date(h.PurchaseDate).yyyymmdd() + '</td>' +
                  '<td class="num">' + formatAmount(h.NoUnits, '', 0) + '</td>' +
                  '<td class="num">' + formatAmount(h.PurchasePricePerUnit, i.Currency, 2) + '</td>' + //Buy Price
                  '<td class="num">' + formatAmount(i.CurrentPrice, i.Currency, 2) + '</td>' + //Current Price
                  '<td class="num">' + formatAmount(bookCost, i.Currency == 'GBp' ? 'GBP' : i.Currency, 0) + '</td>' + //Book Cost
                  '<td class="num">' + formatAmount(currentValue, i.Currency == 'GBp' ? 'GBP' : i.Currency, 0) + '</td>' + //Current Value
                  '<td class="num ' + profitOrLoss + '">' + formatAmount(gain, i.Currency == 'GBp' ? 'GBP' : i.Currency, 0) + '</td>' + //Gain
                  '<td class="num ' + profitOrLoss + '">' + gainPercent.toFixed(1) + '%</td>' + //Gain %
                  '<td class="num">' + formatAmount(currentValueGBP, 'GBP', 2) + '</td>' + //Current Value £
                  '<td class="num">' + formatAmount(currentValueGBP/totalHoldingsValue*100, '', 1) + '%</td>' + //Allocation
                  '<td class="num ' + profitOrLoss + '">' + formatAmount(gainGBP, 'GBP', 0) + '</td>' + //Gain £
                  (marketIsOpen == 0 ? '<td/>' : '<td class="num ' + singleDayProfitOrLoss + '">' + formatAmount(singleDayGainGBP, 'GBP', 0) + '</td>') + //Today's Gain £
                  (marketIsOpen == 0 ? '<td/>' : '<td class="num ' + singleDayProfitOrLoss + '">' + singleDayGainPercent.toFixed(1) + '%</td>') + //Today's Gain %
              '</tr>';
        }
      } else {
        if (i.InstrumentType == 'EQUITY') {
          tbl += '<tr' + (altRow ? ' class="altShareRow"' : '') + '>';
          tbl += '<td>' + i.InstrumentName + '</td>';
          tbl += '<td><a target="_blank" href="https://uk.finance.yahoo.com/quote/' + i.Symbol + '">' + i.Symbol + '</a>' + '</td>';
          tbl += '<td colspan="14">No current price</td></tr>';
        }
      }
    }
    catch (err) {
      console.info("Error adding " + i.Symbol + " to main holdings table: " + err.message);
      tbl+='<tr><td colspan="15">Error adding ' + i.Symbol + ' to table: ' + err.message + '</td></tr>';
    }
  }
  let profitOrLoss = totalValueGBP < totalBookCostGBP ? 'loss' : 'profit';
  let todaysProfitOrLoss = todaysGainGBP < 0 ? 'loss' : 'profit';
  tbl += '</tbody></tfoot><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>';
  tbl += '<tr><td></td><td></td><td></td><td></td><td></td><td></td><td></td>' +
             '<td class="num">' + formatAmount(totalBookCostGBP, 'GBP', 2) + '</td>' +
             '<td></td><td></td>' +
             '<td class="num ' + profitOrLoss + '">' + formatAmount((((totalValueGBP/totalBookCostGBP) - 1) * 100), '', 1) + '%</td>' +
             '<td class="num">' + formatAmount(totalValueGBP, 'GBP', 2) + '</td>' +
             '<td>&nbsp;</td>' + //Allocation
             '<td class="num ' + profitOrLoss + '">' + formatAmount(totalGainGBP, 'GBP', 2) + '</td>' +
             '<td class="num ' + todaysProfitOrLoss + '">' + formatAmount(todaysGainGBP, 'GBP', 2) + '</td>' +
             '<td class="num ' + todaysProfitOrLoss + '">' + formatAmount(((todaysGainGBP/totalBookCostGBP) * 100), '', 1) + '%</td>' +
          '</tr>';
  tbl += '</tfoot></table>';
  $('#holdingsDiv').html(tbl + '<br><div id="divHoldingCurrenciesChart" class="HoldingCurrenciesChart"></div>');



  $("#tblMainHoldings").tablesorter();



  let cd = holdingCurrencies.getChartData();
  function labelFormatter(label, series) {
    return "<div class='pieLabel'>" + label + "<br/>" + Math.round(series.percent) + "%<br>" + formatAmount(series.data[0][1], 'GBP', 0) + "</div>";
  }
  let ops = { series: { pie: { show: true, radius: 1, label: { show: true, radius: 0.7, formatter: labelFormatter } } }, legend: { show: false } };
  if (cd.length) {
    $.plot('#divHoldingCurrenciesChart', cd, ops);
  }
}

function showModalDialog_DeleteHolding(holdingID) {
  let modal = document.getElementById("modalDialog");

  let o = Instruments.GetHolding(holdingID);
  let h = o.holding;
  let i = o.instrument;
  let content = 'Symbol: ' + i.Symbol +
            '<br>Name: ' + i.InstrumentName +
            '<br>Account: ' + h.AccountName +
            '<br>Purchase Date: ' + new Date(h.PurchaseDate).yyyymmdd() +
            '<br>Purchase Price: ' + formatAmount(h.PurchasePricePerUnit, i.Currency, 2) +
            '<br>No Units: ' + formatAmount(h.NoUnits, '', 0) +
            '<br>Cost: ' + formatAmount(h.NoUnits * h.PurchasePricePerUnit, i.Currency, 2) +
            '<br><br>Delete holding?' +
            '<br><br><span class="ok-button">Delete</span>';
  $("#modalContentDiv").html(content);
  
  $("#modalDialog .close").click(function () { modal.style.display = "none"; });

  window.onclick = function (event) { // When the user clicks anywhere outside of the modal, close it
    if (event.target == modal) {
      modal.style.display = "none";
    }
  }

  $("#modalDialog .ok-button").click(function () { modal.style.display = "none"; deleteHolding(holdingID); });
  
  //Display the dialog
  modal.style.display = "block";
}
function deleteHolding(holdingID) {
  //console.info("Delete holding: " + holdingID.toString());

  let o = Instruments.GetHolding(holdingID);
  let h = o.holding;
  let i = o.instrument;

  //this.Data.map(function (item) { return item.Symbol; }).indexOf(Symbol); 
  let idx = i.Holdings.map(function (item) { return item.HoldingID; }).indexOf(holdingID);
  if (idx >= 0) {
    i.Holdings.splice(idx, 1);
  }

  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/DeleteHolding",
    dataType: "json",
    data: JSON.stringify({ HoldingID: holdingID }),
    success: function (response) {
      $("#shareRow" + i.DisplayOrder).html(createSharesTableRow(i));
      createLongChart(i);
      createMidChart(i);
      createShortChart(i);
      createSingleDayChart(i);
    },
    failure: function (response) { console.error("DeleteHolding error: " + JSON.stringify(response)); }
  });
}

function showModalDialog_AddInstrument() {
  let modal = document.getElementById("modalDialog");

  $("#modalTitle").html("Add Instrument");

  let content = '<div>Search text: <input type="text" oninput="searchForYahooInstrument(this.value)"/></div><br>' +
                '<div id="addInstrumentSearchResults"></div>';
  $("#modalContentDiv").html(content);

  $("#modalDialog .close").click(function () { modal.style.display = "none"; });

  window.onclick = function (event) { // When the user clicks anywhere outside of the modal, close it
    if (event.target == modal) {
      modal.style.display = "none";
    }
  }

  //$("#modalDialog .ok-button").click(function () { modal.style.display = "none";  });

  //Display the dialog
  modal.style.display = "block";
}
function searchForYahooInstrument(searchString) {
  if (searchString.length > 1) {
    $.ajax({
      type: "POST",
      contentType: "application/json",
      url: "SharePrices.aspx/SearchYahooFinanceShares",
      dataType: "json",
      data: JSON.stringify({ SearchString: searchString }),
      success: function (response) {
        let results = '';
        if (response.quotes) {
          instrumentSearchResults = response.quotes;
          results = '<table class="instrumentSearchResults"><tr><th>Symbol</th><th>Name</th><th>Exchange</th><th>Type</th></tr>';
          for (let x = 0; x < instrumentSearchResults.length; x++) {
            let q = instrumentSearchResults[x];
            results += '<tr id="searchResult' + x.toString() + '" onclick="selectSearchResult(' + x.toString() + ')"><td>' + q.symbol + '</td>' +
                           '<td>' + (q.longname ? q.longname : q.shortname) + '</td>' +
                           '<td>' + q.exchange + '</td>' +
                           '<td>' + q.typeDisp + '</td>' +
                       '</tr>';
          }
          results += '</table><br><div><span id="addInstrumentOKButton" class="ok-button-disabled">Add Instrument</span></div>';
        }
        $("#addInstrumentSearchResults").html(results);
      },
      failure: function (response) { console.error("searchForYahooInstrument error: " + JSON.stringify(response)); }
    });
  }
}
function selectSearchResult(index) {
  for (let x = 0; x < instrumentSearchResults.length; x++) {
    if (x == index) {
      $("#searchResult" + x.toString()).addClass('selectedSearchResult');
    } else {
      $("#searchResult" + x.toString()).removeClass('selectedSearchResult');
    }
  }
  $("#addInstrumentOKButton").removeClass("ok-button-disabled").addClass("ok-button").unbind().click(function () { $("#modalDialog").css("display", "none"); addInstrument(index) });
}
function addInstrument(index) {
  let i = instrumentSearchResults[index];
  console.info("Add instrument: " + JSON.stringify(i));
  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/AddInstrument",
    dataType: "json",
    data: JSON.stringify({ Symbol: i.symbol, FullName: (i.longname ? i.longname : i.shortname) }),
    success: function (response) {
      if (response.Symbol) {
        Instruments.Data.push(response);
        let i = Instruments.Data[Instruments.Data.length - 1];
        i.Holdings = [];
        i.DailyData = [];
        i.IntradayData = [];
        let lastTR = $('#tblInstruments>tbody>tr').last();
        let cls = (lastTR.hasClass('shareRow')) ? 'altShareRow' : 'shareRow';
        let tr = '<tr class="' + cls + '" id="shareRow' + i.DisplayOrder.toString() + '">' + createSharesTableRow(i) + '</tr>';
        lastTR.after(tr);
      } else {
        console.error("addInstrument response error: " + JSON.stringify(response));
      }
    },
    failure: function (response) { console.error("addInstrument AJAX error: " + JSON.stringify(response)); }
  });
}

function showModalDialog_AddHolding(span) {
  function GetAccountsDropdown() {
    let dd = '<select id="addHoldingAccountName"><option value="">Select account...</option>';
    for (let i = 0; i < Instruments.Accounts.length; i++) {
      dd += '<option value="' + Instruments.Accounts[i].AccountName + '">' + Instruments.Accounts[i].AccountName + '</option>';
    }
    dd += '</select>';
    return dd;
  }

  let symbol = $(span).attr('data-symbol');
  let i = Instruments.GetSymbol(symbol);

  let modal = document.getElementById("modalDialog");
  $("#modalTitle").html("Add Holding");

  let content = '<div><table><tr><th>Instrument Name</th><td>' + i.InstrumentName + '</td></tr>' +
                            '<tr><th>Symbol</th><td>' + symbol + '</td></tr>' +
                            '<tr><th>Account</th><td>' + GetAccountsDropdown() + '</td></tr>' +
                            '<tr><th>Current price</th><td>' + (i.CurrentPrice ? formatAmount(i.CurrentPrice, i.Currency, 2) : '') + '</td></tr>' +
                            '<tr><th>Purchase price / unit</th><td><input type="text" id="newHoldingPrice" value="' + (i.CurrentPrice ? i.CurrentPrice.toString() : '') + '" /></td></tr>' +
                            '<tr><th>No units</th><td><input type="text" id="newHoldingNoUnits" /></td></tr>' +
                            '<tr><th>Total cost</th><td><input type="text" id="newHoldingTotalCost" /></td></tr>' +
                            '<tr><th>Purchase date</th><td><input type="text" id="newHoldingDate" value="' + new Date().yyyymmdd() + '" /></td></tr>' +
                     '</table></div><br>' +
                '<div><span id="newHoldingOKButton" class="ok-button-disabled">OK</span></div>';
  $("#modalContentDiv").html(content);
  $("#newHoldingDate").datepicker({ dateFormat: 'yy-mm-dd', onSelect: function () { ValidateNewHolding(symbol) }});
  $("#modalDialog .close").click(function () { modal.style.display = "none"; });
  window.onclick = function (event) { // When the user clicks anywhere outside of the modal, close it
    if (event.target == modal) {
      modal.style.display = "none";
    }
  }

  $("#newHoldingPrice").bind('input', function () {
    $("#newHoldingOKButton").removeClass("ok-button").addClass("ok-button-disabled").unbind();
    try {
      let noUnits = parseFloat($("#newHoldingNoUnits").val());
      let price = parseFloat($("#newHoldingPrice").val());
      $("#newHoldingTotalCost").val((noUnits * price).toFixed(2));
      ValidateNewHolding(symbol);
    } catch (err) { }
  });
  $("#newHoldingNoUnits").bind('input', function () {
    $("#newHoldingOKButton").removeClass("ok-button").addClass("ok-button-disabled").unbind();
    try {
      let noUnits = parseFloat($("#newHoldingNoUnits").val());
      let price = parseFloat($("#newHoldingPrice").val());
      $("#newHoldingTotalCost").val((noUnits * price).toFixed(2));
      ValidateNewHolding(symbol);
    } catch (err) { }
  });
  $("#newHoldingTotalCost").bind('input', function () {
    $("#newHoldingOKButton").removeClass("ok-button").addClass("ok-button-disabled").unbind();
    try {
      let noUnits = parseFloat($("#newHoldingNoUnits").val());
      let totalCost = parseFloat($("#newHoldingTotalCost").val());
      $("#newHoldingPrice").val((totalCost / noUnits).toFixed(4));
      ValidateNewHolding(symbol);
    } catch (err) { }
  });
  $("#newHoldingDate").bind('input', function () {
    ValidateNewHolding(symbol);
  });
  $("#addHoldingAccountName").bind('input', function () {
    ValidateNewHolding(symbol);
  });

  //Display the dialog
  modal.style.display = "block";
}
function ValidateNewHolding(symbol) {
  $("#newHoldingOKButton").removeClass("ok-button").addClass("ok-button-disabled").unbind();
  try {
    let accountName = $("#addHoldingAccountName").val();
    let noUnits = parseFloat($("#newHoldingNoUnits").val());
    let price = parseFloat($("#newHoldingPrice").val());
    let purchaseDate = $("#newHoldingDate").val();

    if (accountName != '' && accountName != 'Select account...' && noUnits > 0 && price > 0 && purchaseDate != '') {
      if (new Date(purchaseDate) <= new Date()) {
        $("#newHoldingOKButton").removeClass("ok-button-disabled").addClass("ok-button").unbind().bind('click', function () { $("#modalDialog").css("display", "none"); AddHolding(accountName, symbol, noUnits, price, purchaseDate) });
      }
    }
  } catch (err) { }
}
function AddHolding (accountName, symbol, noUnits, purchasePricePerUnit, purchaseDate) {
  console.info("AddHolding: " + accountName + ", " + symbol + ", " + noUnits.toString() + ", " + purchasePricePerUnit.toFixed(4) + ", " + new Date(purchaseDate).yyyymmdd());

  $.ajax({
    type: "POST",
    contentType: "application/json",
    url: "SharePrices.aspx/AddHolding",
    dataType: "json",
    data: JSON.stringify({ AccountName: accountName, Symbol: symbol, NoUnits: noUnits, PurchasePricePerUnit: purchasePricePerUnit, PurchaseDate: new Date(purchaseDate).getTime() }),
    success: function (response) {
      if (response.HoldingID) {
        let i = Instruments.GetSymbol(symbol);
        if (!i.Holdings) i.Holdings = [];
        i.Holdings.push(response);
        let tr = $("#shareRow" + i.DisplayOrder.toString());
        tr.html(createSharesTableRow(i));
        //$('#chartDiv span.addHolding').unbind().bind('click', function () { showModalDialog_AddHolding(this) });
        bindShareRowEvents();
        createLongChart(i);
        createMidChart(i);
        createShortChart(i);
        createSingleDayChart(i);
      } else {
        console.error("addInstrument response error: " + JSON.stringify(response));
      }
    },
    failure: function (response) { console.error("addInstrument AJAX error: " + JSON.stringify(response)); }
  });
}

function zzz_OLD_createSharesTable() {
  function zzz_getHoldingsTable(Instrument) {
    if (Instrument.Holdings.length > 0) {
      let totalUnits = 0;
      let totalPrice = 0;
      let t = '<table class="miniHoldings"><tr><th>Account</th><th>Buy Date</th><th>Price</th><th>Units</th><th>Total</th></tr>';
      for (let i = 0; i < Instrument.Holdings.length; i++) {
        let h = Instrument.Holdings[i];
        totalUnits += h.NoUnits;
        totalPrice += (h.NoUnits * h.PurchasePricePerUnit);
        t += '<tr><td><span class="deletebutton" onclick="showModalDialog_DeleteHolding(' + h.HoldingID + ')">&times;</span>' + h.AccountName + '</td>' +
                 '<td>' + new Date(h.PurchaseDate).yyyymmdd() + '</td>' +
                 '<td class="num">' + formatAmount(h.PurchasePricePerUnit, '', 2) + '</td>' +
                 '<td class="num">' + formatAmount(h.NoUnits, '', 0) + '</td>' +
                 '<td class="num">' + formatAmount(h.NoUnits * h.PurchasePricePerUnit, '', 2) + '</td></tr>';
      }
      Instrument.TotalUnitsHeld = totalUnits;
      Instrument.TotalHoldingsCost = totalPrice;
      Instrument.BreakevenPrice = totalPrice / totalUnits;
      if (Instrument.Holdings.length > 1) {
        t += '<tr><td class="no-border"></td>' +
                 '<td class="no-border">Total</td>' +
                 '<td class="num">' + formatAmount(Instrument.BreakevenPrice, '', 2) + '</td>' +
                 '<td class="num">' + formatAmount(Instrument.TotalUnitsHeld, '', 0) + '</td>' +
                 '<td class="num">' + formatAmount(Instrument.TotalHoldingsCost, '', 2) + '</td>' +
             '</tr>';
      }
      t += '</table>';
      return t;
    } else {
      return '';
    }
  }
  //console.info('createSharesTable()');
  //let tableClass = ($("navCharts").hasClass("activenav")) ? "activetab" : "inactivetab";
  //var tbl = '<table class="' + tableClass + '" id="tblInstruments"><tr><th>Instrument</th>' +
  var tbl = '<table id="tblInstruments"><tr><th>Instrument</th>' +
                                           '<th>YTD</th>' +
                                           '<th>1 Month<button onclick="setMid3Month">3 Month</button>' +
                                                       '<button onclick="setMid2Month">2 Month</button>' +
                                                       '<button onclick="setMid1Month">1 Month</button></th>' +
                                           '<th>1 Week</th>' +
                                           '<th>1 Day</th></tr>';
  for (var rowNo = 0; rowNo < Instruments.Data.length; rowNo++) {
    let i = Instruments.Data[rowNo];
    if (i.DisplayOrder) {
      let cls = (rowNo % 2) ? 'altShareRow' : 'shareRow';
      let tr = '<tr class="' + cls + '"><td>' + i.InstrumentName + '<br>' +
                                       '<a target="_blank" href="https://uk.finance.yahoo.com/quote/' + i.Symbol + '">' + i.Symbol + '</a><div class="spacer"></div>' +
                                       getHoldingsTable(i) +
                                       '<div class="price-summary" id="divCurrentValue' + i.DisplayOrder + '" />'+
                                     '</td>' +
                                     '<td><div class="LongChart" id="divLongChart' + i.DisplayOrder.toString() + '"></div><div class="LongSummary" id="divLongSummary' + i.DisplayOrder.toString() + '" /></td>' +
                                     '<td><div class="MidChart" id="divMidChart' + i.DisplayOrder.toString() + '"></div><div class="MidSummary" id="divMidSummary' + i.DisplayOrder.toString() + '" /></td>' +
                                     '<td><div class="ShortChart" id="divShortChart' + i.DisplayOrder.toString() + '"></div><div class="ShortSummary" id="divShortSummary' + i.DisplayOrder.toString() + '" /></td>' +
                                     '<td><div class="DayChart" id="divDayChart' + i.DisplayOrder.toString() + '"></div><div class="DaySummary" id="divDaySummary' + i.DisplayOrder.toString() + '" /></td>' +
             '</tr>';
      tbl += tr;
    }
  }
  tbl += '</table>';
  $('#chartDiv').html(tbl);
  getAllLocalDailyData();
};